@@ -50,8 +50,6 @@ zig_keywords = {
     'anyframe',
     'anytype',
     'asm',
-    'async',
-    'await',
     'break',
     'callconv',
     'catch',
@@ -88,7 +86,6 @@ zig_keywords = {
     'try',
     'union',
     'unreachable',
-    'usingnamespace',
     'var',
     'volatile',
     'while',
@@ -599,9 +596,10 @@ type_tag_handlers = {
     'slice_const_u8_sentinel_0': lambda payload: '[:0]const u8',
     'fn_noreturn_no_args': lambda payload: 'fn() noreturn',
     'fn_void_no_args': lambda payload: 'fn() void',
-    'fn_naked_noreturn_no_args': lambda payload: 'fn() callconv(.Naked) noreturn',
-    'fn_ccc_void_no_args': lambda payload: 'fn() callconv(.C) void',
-    'single_const_pointer_to_comptime_int': lambda payload: '*const comptime_int',
+    'fn_naked_noreturn_no_args': lambda payload: 'fn() callconv(.naked) noreturn',
+    'fn_ccc_void_no_args': lambda payload: 'fn() callconv(.c) void',
+    'ptr_usize': lambda payload: '*usize',
+    'ptr_const_comptime_int': lambda payload: '*const comptime_int',
     'manyptr_u8': lambda payload: '[*]u8',
     'manyptr_const_u8': lambda payload: '[*]const u8',
     'manyptr_const_u8_sentinel_0': lambda payload: '[*:0]const u8',
